"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.treeFromValue = treeFromValue;
exports.shapePath = shapePath;
exports.pathExistsInTree = pathExistsInTree;
exports.updateAtPath = updateAtPath;
exports.shapedArrayChild = shapedArrayChild;
exports.shapedArrayChildren = shapedArrayChildren;
exports.shapedObjectChild = shapedObjectChild;
exports.forgetShape = forgetShape;
exports.mapRoot = mapRoot;
exports.dangerouslyReplaceObjectChild = dangerouslyReplaceObjectChild;
exports.dangerouslyReplaceArrayChild = dangerouslyReplaceArrayChild;
exports.dangerouslySetChildren = dangerouslySetChildren;
exports.mapShapedTree = mapShapedTree;
exports.foldMapShapedTree = foldMapShapedTree;
exports.getRootData = getRootData;
exports.rootPath = void 0;

var _tree = require("./tree");

var _invariant = _interopRequireDefault(require("./utils/invariant"));

var _array = require("./utils/array");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var rootPath = function rootPath() {
  return [];
}; // Take shape from value, data from nodeData


exports.rootPath = rootPath;

function treeFromValue(value, nodeData) {
  if (Array.isArray(value)) {
    return {
      type: "array",
      data: nodeData,
      children: value.map(function (child) {
        return treeFromValue(child, nodeData);
      })
    };
  }

  if (value instanceof Object) {
    var objectValue = value;
    return {
      type: "object",
      data: nodeData,
      children: Object.keys(objectValue).reduce(function (children, k) {
        return _objectSpread({}, children, _defineProperty({}, k, treeFromValue(objectValue[k], nodeData)));
      }, {})
    };
  }

  return {
    type: "leaf",
    data: nodeData
  };
}

function shapePath(data, path) {
  if (path.length === 0) {
    return path;
  }

  var _path = _toArray(path),
      firstPart = _path[0],
      restParts = _path.slice(1);

  if (firstPart.type === "object" && Object.hasOwnProperty.call(data, firstPart.key)) {
    // $FlowFixMe: This is safe
    var restPath = shapePath(data[firstPart.key], restParts);

    if (restPath === null) {
      return null;
    }

    return [firstPart].concat(_toConsumableArray(restPath));
  } else if (firstPart.type === "array" && Array.isArray(data) && firstPart.index < data.length) {
    var _restPath = shapePath(data[firstPart.index], restParts);

    if (_restPath === null) {
      return null;
    }

    return [firstPart].concat(_toConsumableArray(_restPath));
  }

  return null;
}

function pathExistsInTree(path, tree) {
  // This function is equivalent to:
  // try {
  //   updateAtPath(path, x => x, tree);
  // } catch {
  //   return false;
  // }
  // return true;
  // }
  if (path.length === 0 && tree.type != null) {
    return true;
  }

  var _path2 = _toArray(path),
      first = _path2[0],
      rest = _path2.slice(1);

  if (tree.type === "leaf") {
    return false;
  }

  if (tree.type === "array") {
    if (first.type !== "array") {
      return false;
    }

    if (!(0 <= first.index && first.index < tree.children.length)) {
      return false;
    }

    return pathExistsInTree(rest, tree.children[first.index]);
  }

  if (tree.type === "object") {
    if (first.type !== "object") {
      return false;
    }

    if (!(first.key in tree.children)) {
      return false;
    }

    return pathExistsInTree(rest, tree.children[first.key]);
  }

  throw new Error("unreachable");
}

function updateAtPath(path, updater, tree) {
  if (path.length === 0) {
    if (tree.type === "object") {
      return {
        type: "object",
        data: updater(tree.data),
        children: tree.children
      };
    }

    if (tree.type === "array") {
      return {
        type: "array",
        data: updater(tree.data),
        children: tree.children
      };
    }

    return {
      type: "leaf",
      data: updater(tree.data)
    };
  }

  var _path3 = _toArray(path),
      firstStep = _path3[0],
      restStep = _path3.slice(1);

  if (tree.type === "leaf") {
    throw new Error("Theres more key, but not more Tree to match it against");
  }

  if (tree.type === "array") {
    (0, _invariant.default)(firstStep.type === "array", "Trying to take a non-array path into an array");
    (0, _invariant.default)(0 <= firstStep.index && firstStep.index < tree.children.length, "Tried to take path index ".concat(firstStep.index, " but array from tree has length ").concat(tree.children.length));
    var newChild = updateAtPath(restStep, updater, tree.children[firstStep.index]); // $FlowFixMe(zach): I think this is safe, might need GADTs for the type checker to understand why

    return dangerouslyReplaceArrayChild(firstStep.index, newChild, tree);
  }

  if (tree.type === "object") {
    (0, _invariant.default)(firstStep.type === "object", "Trying to take a non-object path into an object");
    var nextTree = tree.children[firstStep.key];
    (0, _invariant.default)(nextTree !== undefined, "Tried to take path key ".concat(firstStep.key, " but it isn't present on object in tree"));

    var _newChild = updateAtPath(restStep, updater, nextTree); // $FlowFixMe(zach): I think this is safe, might need GADTs for the type checker to understand why


    return dangerouslyReplaceObjectChild(firstStep.key, _newChild, tree);
  }

  throw new Error("unreachable");
}

function shapedArrayChild(index, tree) {
  (0, _invariant.default)(tree.type === "array", "Tried to get an array child of a non-array node");
  return tree.children[index];
}

function shapedArrayChildren(tree) {
  (0, _invariant.default)(tree.type === "array", "Tried to get an array children of a non-array node");
  return tree.children.map(function (x) {
    return x;
  });
} // TODO(zach): not sure what to do about this output variable


function shapedObjectChild(key, tree) {
  (0, _invariant.default)(tree.type === "object", "Tried to get object child of a non-object node");
  return tree.children[key];
}

function forgetShape(tree) {
  return tree;
}

function mapRoot(f, tree) {
  if (tree.type === "object") {
    return {
      type: "object",
      data: f(tree.data),
      children: tree.children
    };
  }

  if (tree.type === "array") {
    return {
      type: "array",
      data: f(tree.data),
      children: tree.children
    };
  }

  return {
    type: "leaf",
    data: f(tree.data)
  };
} // Do not use this unless you really know what you are doing
// It is unsafe, and intended to bypass some checks


function dangerouslyReplaceObjectChild(key, child, tree) {
  (0, _invariant.default)(tree.type === "object", "Tried to replace child of a non-object node");
  return {
    type: "object",
    data: tree.data,
    children: _objectSpread({}, tree.children, _defineProperty({}, key, child))
  };
} // Do not use this unless you really know what you are doing
// It is unsafe, and intended to bypass some checks


function dangerouslyReplaceArrayChild(index, child, tree) {
  (0, _invariant.default)(tree.type === "array", "Tried to replace child of a non-array node");
  return {
    type: "array",
    data: tree.data,
    children: (0, _array.replaceAt)(index, child, tree.children)
  };
} // Do not use this unless you really know what you are doing
// It is intended to bypass some checks when modifying arrays


function dangerouslySetChildren(children, tree) {
  (0, _invariant.default)(tree.type === "array", "Tried to set children of a non-array node");
  return {
    type: "array",
    data: tree.data,
    children: children.map(forgetShape)
  };
} // Mapping doesn't change the shape


function mapShapedTree(f, tree) {
  return (0, _tree.mapTree)(f, tree);
} // Fold a tree inorder


function foldMapShapedTree(mapper, mempty, mappend, tree) {
  return (0, _tree.foldMapTree)(mapper, mempty, mappend, tree);
}

function getRootData(tree) {
  return tree.data;
}