"use strict";

var React = _interopRequireWildcard(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _Form = _interopRequireDefault(require("../Form"));

var _ArrayField = _interopRequireDefault(require("../ArrayField"));

var _TestField = _interopRequireWildcard(require("./TestField"));

var _TestForm = _interopRequireDefault(require("./TestForm"));

var _tools = require("./tools");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("ArrayField", function () {
  describe("is a field", function () {
    it("ensures that the link inner type matches the type of the validation", function () {
      var formState = (0, _tools.mockFormState)(["one", "two", "three"]);
      var link = (0, _tools.mockLink)(formState); // $ExpectError

      React.createElement(_ArrayField.default, {
        link: link,
        validation: function validation(_e) {
          return [];
        }
      }, function () {
        return null;
      });
      React.createElement(_ArrayField.default, {
        link: link,
        validation: function validation(_e) {
          return [];
        }
      }, function () {
        return null;
      });
    });
    it("Registers and unregisters for validation", function () {
      var formState = (0, _tools.mockFormState)([]);
      var link = (0, _tools.mockLink)(formState);
      var unregister = jest.fn();
      var registerValidation = jest.fn(function () {
        return {
          replace: jest.fn(),
          unregister: unregister
        };
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
        registerValidation: registerValidation
      }, React.createElement(_ArrayField.default, {
        link: link,
        validation: function validation() {
          return [];
        }
      }, function () {
        return null;
      })));

      expect(registerValidation).toBeCalledTimes(1);
      renderer.unmount();
      expect(unregister).toBeCalledTimes(1);
    });
    it("calls replace when changing the validation function", function () {
      var replace = jest.fn();
      var registerValidation = jest.fn(function () {
        return {
          replace: replace,
          unregister: jest.fn()
        };
      });

      function Component() {
        return React.createElement(_TestForm.default, {
          registerValidation: registerValidation
        }, React.createElement(_ArrayField.default, {
          link: (0, _tools.mockLink)((0, _tools.mockFormState)(["hello", "world"])),
          validation: function validation() {
            return [];
          }
        }, function () {
          return null;
        }));
      }

      var renderer = _reactTestRenderer.default.create(React.createElement(Component, null));

      expect(registerValidation).toBeCalledTimes(1);
      renderer.update(React.createElement(Component, null));
      expect(replace).toBeCalledTimes(1);
    });
    it("Passes additional information to its render function", function () {
      var formState = (0, _tools.mockFormState)(["value"]); // $FlowFixMe

      formState[1].data.errors = {
        external: ["An external error"],
        client: ["A client error"]
      };
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
        link: link
      }, renderFn));

      expect(renderFn).toHaveBeenCalled();
      expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.anything(), expect.objectContaining({
        touched: false,
        changed: false,
        shouldShowErrors: expect.anything(),
        unfilteredErrors: expect.arrayContaining(["An external error", "A client error"]),
        valid: false,
        asyncValidationInFlight: false,
        value: ["value"]
      }));
    });
  });
  describe("An ArrayField disassembles an array into each item", function () {
    it("breaks apart a link into an array of links", function () {
      var formStateValue = ["one", "two", "three"];
      var formState = (0, _tools.mockFormState)(formStateValue);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
        link: link
      }, renderFn));

      expect(renderFn).toHaveBeenCalled();
      var arrayLinks = renderFn.mock.calls[0][0];
      expect(arrayLinks.length).toBe(3);
      arrayLinks.forEach(_tools.expectLink);
    });
    it("has the correct type for the links object", function () {
      var formState = (0, _tools.mockFormState)(["one", "two", "three"]);
      var link = (0, _tools.mockLink)(formState);
      React.createElement(_ArrayField.default, {
        link: link
      }, function (links) {
        console.log(links);
        return null;
      });
      React.createElement(_ArrayField.default, {
        link: link
      }, function (links) {
        console.log(links);
        return null;
      });
    });
    it("validates new values from children and passes result to onChange", function () {
      var formState = (0, _tools.mockFormState)(["one", "two", "three"]);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });
      var applyChangeToNode = jest.fn(function (path, formState) {
        return formState;
      });

      _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
        applyChangeToNode: applyChangeToNode
      }, React.createElement(_ArrayField.default, {
        link: link
      }, renderFn)));

      expect(applyChangeToNode).toHaveBeenCalledTimes(0);
      expect(link.onChange).toHaveBeenCalledTimes(0); // call a child's onChange

      var arrayLinks = renderFn.mock.calls[0][0];
      var newElementFormState = (0, _tools.mockFormState)("newTwo");
      arrayLinks[1].onChange(newElementFormState);
      expect(applyChangeToNode).toHaveBeenCalledTimes(1);
      expect(applyChangeToNode).toHaveBeenCalledWith([], [["one", "newTwo", "three"], expect.anything()]);
      expect(link.onChange).toHaveBeenCalledTimes(1);
      expect(link.onChange).toHaveBeenCalledWith([["one", "newTwo", "three"], formState[1]]);
    });
    it("calls onBlur when a child is blurred", function () {
      var formStateValue = ["one", "two", "three"];
      var formState = (0, _tools.mockFormState)(formStateValue);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
        link: link
      }, renderFn));

      var arrayLinks = renderFn.mock.calls[0][0];
      var newElementTree = (0, _tools.mockFormState)("")[1];
      arrayLinks[0].onBlur(newElementTree);
      expect(link.onBlur).toHaveBeenCalled();
      var newArrayTree = link.onBlur.mock.calls[0][0];
      expect(newArrayTree.data.meta).toMatchObject({
        touched: true,
        changed: false
      });
      expect(newArrayTree.children[0]).toBe(newElementTree);
    });
    it("calls its validation when a child changes", function () {
      var renderFn = jest.fn(function () {
        return null;
      });
      var validation = jest.fn(function () {
        return ["This is an error"];
      });

      _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: ["one", "two", "three"]
      }, function (link) {
        return React.createElement(_ArrayField.default, {
          link: link,
          validation: validation
        }, renderFn);
      }));

      expect(validation).toHaveBeenCalledTimes(1);
      var arrayLinks = renderFn.mock.calls[0][0];
      var newElementFormState = (0, _tools.mockFormState)("newOne");
      arrayLinks[0].onChange(newElementFormState);
      expect(validation).toHaveBeenCalledTimes(2);
      expect(validation).toHaveBeenLastCalledWith(["newOne", "two", "three"]);
    });
  });
  describe("An ArrayField provides a way to modify the array", function () {
    describe("addField", function () {
      it("exposes addField to add an entry", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          addField: expect.any(Function)
        }), expect.anything());
      });
      it("validates after entry is added", function () {
        var renderFn = jest.fn(function () {
          return null;
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$ = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$[0],
            addField = _renderFn$mock$calls$[1].addField;

        addField(0, "zero");
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["zero", "one", "two", "three"]);
      });
    });
    describe("removeField", function () {
      it("exposes removeField to remove an entry", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          removeField: expect.any(Function)
        }), expect.anything());
      });
      it("validates after entry is removed", function () {
        var elementValidation = jest.fn(function () {
          return ["an element error"];
        });
        var renderFn = jest.fn(function (links) {
          return links.map(function (link, i) {
            return React.createElement(_TestField.default, {
              key: i,
              link: link,
              validation: elementValidation
            });
          });
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$2 = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$2[0],
            removeField = _renderFn$mock$calls$2[1].removeField;

        removeField(1);
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["one", "three"]);
      });
    });
    describe("moveField", function () {
      it("exposes moveField to move an entry", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          moveField: expect.any(Function)
        }), expect.anything());
      });
      it("validates after the entry is moved", function () {
        var renderFn = jest.fn(function () {
          return null;
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$3 = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$3[0],
            moveField = _renderFn$mock$calls$3[1].moveField;

        moveField(2, 1);
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["one", "three", "two"]);
      });
    });
    describe("addFields", function () {
      it("exposes addFields to add an entry", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          addFields: expect.any(Function)
        }), expect.anything());
      });
      it("validates after fields are added", function () {
        var renderFn = jest.fn(function () {
          return null;
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$4 = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$4[0],
            addFields = _renderFn$mock$calls$4[1].addFields;

        addFields([[0, ["negative one", "zero"]], [3, ["four", "five"]]]);
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["negative one", "zero", "one", "two", "three", "four", "five"]);
      });
    });
    describe("filterFields", function () {
      it("exposes filterFields to filter entries", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          filterFields: expect.any(Function)
        }), expect.anything());
      });
      it("validates after fields are filtered", function () {
        var renderFn = jest.fn(function () {
          return null;
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$5 = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$5[0],
            filterFields = _renderFn$mock$calls$5[1].filterFields; // remove numbers without "o" and the fourth element


        filterFields(function (v, i) {
          return v.indexOf("o") !== -1 && i !== 3;
        });
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["one", "two"]);
      });
    });
    describe("modifyFields", function () {
      it("exposes modifyFields to add and remove entries atomically", function () {
        var formStateValue = ["one", "two", "three"];
        var formState = (0, _tools.mockFormState)(formStateValue);
        var link = (0, _tools.mockLink)(formState);
        var renderFn = jest.fn(function () {
          return null;
        });

        _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
          link: link
        }, renderFn));

        expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
          modifyFields: expect.any(Function)
        }), expect.anything());
      });
      it("validates after fields are modified", function () {
        var renderFn = jest.fn(function () {
          return null;
        });
        var validation = jest.fn(function () {
          return ["an error"];
        });

        _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: ["one", "two", "three"]
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            validation: validation,
            link: link
          }, renderFn);
        }));

        expect(validation).toHaveBeenCalledTimes(1);

        var _renderFn$mock$calls$6 = _slicedToArray(renderFn.mock.calls[0], 2),
            _ = _renderFn$mock$calls$6[0],
            modifyFields = _renderFn$mock$calls$6[1].modifyFields;

        modifyFields({
          insertSpans: [[0, ["start"]], [2, ["middle", "content"]]],
          filterPredicate: function filterPredicate(v) {
            return v !== "one";
          }
        });
        expect(validation).toHaveBeenCalledTimes(2);
        expect(validation).toHaveBeenLastCalledWith(["start", "two", "middle", "content", "three"]);
      });
    });
  });
  describe("customChange", function () {
    it("allows sibling fields to be overwritten", function () {
      var formStateInner = ["one", "two", "three"];
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });
      var validation = jest.fn(function () {
        return ["This is an error"];
      });
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return ["uno", "dos", "tres"];
      });

      _reactTestRenderer.default.create(React.createElement(_TestForm.default, null, React.createElement(_ArrayField.default, {
        link: link,
        validation: validation,
        customChange: customChange
      }, renderFn)));

      var arrayLinks = renderFn.mock.calls[0][0]; // call the child onChange

      var newChildMeta = (0, _tools.mockFormState)("zwei");
      arrayLinks[1].onChange(newChildMeta); // customChange should be called with the correct args

      expect(customChange).toHaveBeenCalledTimes(1);
      expect(customChange).toHaveBeenCalledWith(["one", "two", "three"], ["one", "zwei", "three"]); // onChange should be called with the result of customChange

      expect(link.onChange).toHaveBeenCalledTimes(1);
      expect(link.onChange).toHaveBeenCalledWith([["uno", "dos", "tres"], expect.anything()]);
    });
    it("can return null to signal there was no custom change", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return null;
      });
      var renderFn = jest.fn(function () {
        return null;
      });
      var validation = jest.fn(function () {
        return ["an error"];
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: ["one", "two", "three"]
      }, function (link) {
        return React.createElement(_ArrayField.default, {
          validation: validation,
          link: link,
          customChange: customChange
        }, renderFn);
      }));

      var arrayLinks = renderFn.mock.calls[0][0]; // call the child onChange

      var newChildMeta = (0, _tools.mockFormState)("zwei");
      arrayLinks[1].onChange(newChildMeta);
      expect(customChange).toHaveBeenCalledTimes(1); // onChange should be called with the result of customChange

      var link = renderer.root.findByType(_ArrayField.default).instance.props.link;
      expect(link.formState).toEqual([["one", "zwei", "three"], expect.anything()]); // Validated the result of customChange

      expect(validation).toHaveBeenCalledTimes(2);
      expect(validation.mock.calls[1][0]).toEqual(["one", "zwei", "three"]);
    });
    it("doesn't break validations for child fields", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return ["1", "2"];
      });
      var childValidation = jest.fn(function () {
        return ["This is an error"];
      });
      var parentValidation = jest.fn(function () {
        return ["This is an error from the parent"];
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: ["1", "2"]
      }, function (link) {
        return React.createElement(_ArrayField.default, {
          link: link,
          customChange: customChange,
          validation: parentValidation
        }, function (links) {
          return React.createElement(React.Fragment, null, links.map(function (link, i) {
            return React.createElement(_TestField.default, {
              key: i,
              link: link,
              validation: childValidation
            });
          }));
        });
      })); // after mount, validate everything


      expect(parentValidation).toHaveBeenCalledTimes(1);
      expect(childValidation).toHaveBeenCalledTimes(2); // Now change one of the values

      parentValidation.mockClear();
      childValidation.mockClear();
      var inner = renderer.root.findAllByType(_TestField.TestInput)[0];
      inner.instance.change("zach"); // Note that child validation will be called 5 times:
      // * Once in response to the change
      // * Twice more from the arrayEquals test in replaceValidation
      // * Twice more from recomputeErrorsAtPathAndRender

      expect(parentValidation).toHaveBeenCalledTimes(1);
      expect(childValidation).toHaveBeenCalledTimes(1 + 2 + 2);
      var link = renderer.root.findByType(_ArrayField.default).instance.props.link;
      expect(link.formState).toEqual([["1", "2"], expect.anything()]);
    });
    it("doesn't create a new instance (i.e. remount)", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return ["uno", "dos"];
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_ArrayField.default, {
        link: (0, _tools.mockLink)((0, _tools.mockFormState)(["1", "2"])),
        customChange: customChange
      }, function (links) {
        return React.createElement(_TestField.default, {
          link: links[0]
        });
      }));

      var testInstance = renderer.root.findAllByType(_TestField.TestInput)[0].instance; // now trigger a customChange, which used to cause a remount

      testInstance.change("hi");
      expect(customChange).toHaveBeenCalledTimes(1); // but we no longer cause a remount, so the instances should be the same

      var nextTestInstance = renderer.root.findAllByType(_TestField.TestInput)[0].instance; // Using Object.is here because toBe hangs as the objects are
      // self-referential and thus not printable

      expect(Object.is(testInstance, nextTestInstance)).toBe(true);
    });
    describe("length", function () {
      function expectLengthChangeToNotThrow(from, to) {
        var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
          initialValue: from
        }, function (link) {
          return React.createElement(_ArrayField.default, {
            link: link,
            customChange: function customChange() {
              return to;
            }
          }, function (links) {
            return links.map(function (link, i) {
              return React.createElement(_TestField.default, {
                link: link,
                key: i
              });
            });
          });
        }));

        var inner = renderer.root.findAllByType(_TestField.TestInput)[0];
        expect(function () {
          inner.instance.change("z");
        }).not.toThrow();
      }

      it("length can increase", function () {
        expectLengthChangeToNotThrow(["a"], ["a", "b", "c"]);
      });
      it("length can decrease", function () {
        expectLengthChangeToNotThrow(["a", "b", "c"], ["a"]);
      });
    });
  });
});