"use strict";

var React = _interopRequireWildcard(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _Field = _interopRequireDefault(require("../Field"));

var _tools = require("./tools");

var _TestField = _interopRequireWildcard(require("./TestField"));

var _TestForm = _interopRequireDefault(require("./TestForm"));

var _shapedTree = require("../shapedTree");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe("Field", function () {
  it("ensures that the link inner type matches the type of the validation", function () {
    var formState = (0, _tools.mockFormState)("Hello world.");
    var link = (0, _tools.mockLink)(formState); // $ExpectError

    React.createElement(_Field.default, {
      link: link,
      validation: function validation(_e) {
        return [];
      }
    }, function () {
      return null;
    });
    React.createElement(_Field.default, {
      link: link,
      validation: function validation(_e) {
        return [];
      }
    }, function () {
      return null;
    });
  });
  it("registers and unregisters for validation", function () {
    var formState = (0, _tools.mockFormState)("Hello world.");
    var link = (0, _tools.mockLink)(formState);
    var unregister = jest.fn();
    var registerValidation = jest.fn(function () {
      return {
        replace: jest.fn(),
        unregister: unregister
      };
    });

    var renderer = _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
      registerValidation: registerValidation
    }, React.createElement(_Field.default, {
      link: link,
      validation: jest.fn(function () {
        return [];
      })
    }, jest.fn(function () {
      return null;
    }))));

    expect(registerValidation).toBeCalledTimes(1);
    renderer.unmount();
    expect(unregister).toBeCalledTimes(1);
  });
  it("calls replace when changing the validation function", function () {
    var replace = jest.fn();
    var registerValidation = jest.fn(function () {
      return {
        replace: replace,
        unregister: jest.fn()
      };
    });

    function Component() {
      return React.createElement(_TestForm.default, {
        registerValidation: registerValidation
      }, React.createElement(_Field.default, {
        link: (0, _tools.mockLink)((0, _tools.mockFormState)("Hello world.")),
        validation: function validation() {
          return [];
        }
      }, function () {
        return null;
      }));
    }

    var renderer = _reactTestRenderer.default.create(React.createElement(Component, null));

    expect(registerValidation).toBeCalledTimes(1);
    renderer.update(React.createElement(Component, null));
    expect(replace).toBeCalledTimes(1);
  });
  it("validates new values and passes result to onChange", function () {
    var formState = (0, _tools.mockFormState)("Hello world.");
    var link = (0, _tools.mockLink)(formState);
    var applyChangeToNode = jest.fn(function (path, formState) {
      return formState;
    });

    var renderer = _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
      applyChangeToNode: applyChangeToNode
    }, React.createElement(_TestField.default, {
      link: link
    })));

    var inner = renderer.root.findByType(_TestField.TestInput);
    expect(applyChangeToNode).toHaveBeenCalledTimes(0);
    expect(link.onChange).toHaveBeenCalledTimes(0);
    inner.instance.change("You've got mail");
    expect(applyChangeToNode).toHaveBeenCalledTimes(1);
    expect(applyChangeToNode).toHaveBeenCalledWith([], ["You've got mail", expect.anything()]);
    expect(link.onChange).toHaveBeenCalledTimes(1);
    expect(link.onChange).toHaveBeenCalledWith(["You've got mail", formState[1]]);
  });
  it("calls the link onBlur with correct meta", function () {
    var formState = (0, _tools.mockFormState)("");
    var link = (0, _tools.mockLink)(formState);

    var renderer = _reactTestRenderer.default.create(React.createElement(_TestField.default, {
      link: link
    }));

    var inner = renderer.root.findByType(_TestField.TestInput);
    expect(link.onBlur).toHaveBeenCalledTimes(0);
    inner.instance.blur();
    expect(link.onBlur).toHaveBeenCalledTimes(1);
    var tree = link.onBlur.mock.calls[0][0];
    expect(tree.data).toMatchObject({
      meta: {
        touched: true,
        changed: false,
        succeeded: false
      }
    });
  });
  it("flattens errors for the inner component", function () {
    var formState = (0, _tools.mockFormState)("");
    formState[1] = (0, _shapedTree.mapRoot)(function (oldRoot) {
      return _objectSpread({}, oldRoot, {
        errors: {
          client: ["Some", "client", "errors"],
          external: ["External errors", "go here"]
        }
      });
    }, formState[1]);
    var link = (0, _tools.mockLink)(formState);

    var renderer = _reactTestRenderer.default.create(React.createElement(_TestField.default, {
      link: link
    }));

    var inner = renderer.root.findByType(_TestField.TestInput);
    expect(inner.props.errors).toEqual(["Some", "client", "errors", "External errors", "go here"]);
  });
  it("Passes value of the right type to its render function", function () {
    var formState = (0, _tools.mockFormState)("Hello there");
    var link = (0, _tools.mockLink)(formState);
    React.createElement(_Field.default, {
      link: link
    }, function (_value) {
      return null;
    });
    React.createElement(_Field.default, {
      link: link
    }, function (_value) {
      return null;
    });
  });
  it("Passes onChange of the right type to its render function", function () {
    var formState = (0, _tools.mockFormState)("Hello there");
    var link = (0, _tools.mockLink)(formState);
    React.createElement(_Field.default, {
      link: link
    }, function (_value, _errors, _onChange) {
      return null;
    }); // $ExpectError

    React.createElement(_Field.default, {
      link: link
    }, function (_value, _errors, _onChange) {
      return null;
    });
    React.createElement(_Field.default, {
      link: link
    }, function (_value, _errors, _onChange) {
      return null;
    });
  });
  it("Passes additional information to its render function", function () {
    var formState = (0, _tools.mockFormState)(10); // $FlowFixMe

    formState[1].data.errors = {
      external: ["An external error"],
      client: ["A client error"]
    };
    var link = (0, _tools.mockLink)(formState);
    var renderFn = jest.fn(function () {
      return null;
    });

    _reactTestRenderer.default.create(React.createElement(_Field.default, {
      link: link
    }, renderFn));

    expect(renderFn).toHaveBeenCalled();
    expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.anything(), expect.anything(), expect.anything(), expect.objectContaining({
      touched: false,
      changed: false,
      shouldShowErrors: expect.anything(),
      unfilteredErrors: expect.arrayContaining(["An external error", "A client error"]),
      valid: false,
      asyncValidationInFlight: false,
      value: 10
    }));
  });
});