"use strict";

var React = _interopRequireWildcard(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _Form = _interopRequireWildcard(require("../Form"));

var _ObjectField = _interopRequireDefault(require("../ObjectField"));

var _tools = require("./tools");

var _TestField = _interopRequireWildcard(require("./TestField"));

var _TestForm = _interopRequireDefault(require("./TestForm"));

var _LinkTap = _interopRequireDefault(require("../testutils/LinkTap"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe("ObjectField", function () {
  describe("Sneaky hacks", function () {
    it("exposes FormContext as its contextType", function () {
      expect(_ObjectField.default._contextType).toBe(_Form.FormContext);
    });
  });
  describe("is a field", function () {
    it("ensures that the link inner type matches the type of the validation", function () {
      var formStateInner = {
        string: "hello",
        number: 42
      };
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState); // $ExpectError

      React.createElement(_ObjectField.default, {
        link: link,
        validation: function validation(_e) {
          return [];
        }
      }, function () {
        return null;
      }); // $ExpectError

      React.createElement(_ObjectField.default, {
        link: link,
        validation: function validation(_e) {
          return [];
        }
      }, function () {
        return null;
      });
      React.createElement(_ObjectField.default, {
        link: link,
        validation: function validation(_e) {
          return [];
        }
      }, function () {
        return null;
      });
    });
    it("Registers and unregisters for validation", function () {
      var formState = (0, _tools.mockFormState)({
        inner: "value"
      });
      var link = (0, _tools.mockLink)(formState);
      var unregister = jest.fn();
      var registerValidation = jest.fn(function () {
        return {
          replace: jest.fn(),
          unregister: unregister
        };
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
        registerValidation: registerValidation
      }, React.createElement(_ObjectField.default, {
        link: link,
        validation: jest.fn(function () {
          return [];
        })
      }, jest.fn(function () {
        return null;
      }))));

      expect(registerValidation).toBeCalledTimes(1);
      renderer.unmount();
      expect(unregister).toBeCalledTimes(1);
    });
    it("calls replace when changing the validation function", function () {
      var replace = jest.fn();
      var registerValidation = jest.fn(function () {
        return {
          replace: replace,
          unregister: jest.fn()
        };
      });

      function Component() {
        return React.createElement(_TestForm.default, {
          registerValidation: registerValidation
        }, React.createElement(_ObjectField.default, {
          link: (0, _tools.mockLink)((0, _tools.mockFormState)({
            hello: "world"
          })),
          validation: function validation() {
            return [];
          }
        }, function () {
          return null;
        }));
      }

      var renderer = _reactTestRenderer.default.create(React.createElement(Component, null));

      expect(registerValidation).toBeCalledTimes(1);
      renderer.update(React.createElement(Component, null));
      expect(replace).toBeCalledTimes(1);
    });
    it("Passes additional information to its render function", function () {
      var formState = (0, _tools.mockFormState)({
        inner: "value"
      }); // $FlowFixMe

      formState[1].data.errors = {
        external: ["An external error"],
        client: ["A client error"]
      };
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ObjectField.default, {
        link: link
      }, renderFn));

      expect(renderFn).toHaveBeenCalled();
      expect(renderFn).toHaveBeenCalledWith(expect.anything(), expect.objectContaining({
        touched: false,
        changed: false,
        shouldShowErrors: expect.anything(),
        unfilteredErrors: expect.arrayContaining(["An external error", "A client error"]),
        valid: false,
        asyncValidationInFlight: false,
        value: {
          inner: "value"
        }
      }));
    });
  });
  describe("An ObjectField disassembles an object into its fields", function () {
    it("breaks apart a link to an object into an object of links", function () {
      var formStateInner = {
        string: "hello",
        number: 42
      };
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ObjectField.default, {
        link: link
      }, renderFn));

      expect(renderFn).toHaveBeenCalled();
      var objectLinks = renderFn.mock.calls[0][0];
      expect(Object.keys(objectLinks)).toEqual(Object.keys(formStateInner));
      Object.keys(formStateInner).forEach(function (k) {
        expect(objectLinks).toHaveProperty(k);
        var link = objectLinks[k];
        (0, _tools.expectLink)(link);
      });
    });
    it("has the correct type for the links object", function () {
      var formStateInner = {
        string: "hello",
        number: 42
      };
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState);
      React.createElement(_ObjectField.default, {
        link: link
      }, function (links) {
        console.log(links);
        return null;
      });
      React.createElement(_ObjectField.default, {
        link: link
      }, function (links) {
        console.log(links);
        return null;
      });
      React.createElement(_ObjectField.default, {
        link: link
      }, function (links) {
        console.log(links);
        return null;
      });
    });
    it("validates new values from children and passes result to onChange", function () {
      var formState = (0, _tools.mockFormState)({
        string: "hello",
        number: 42
      });
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });
      var applyChangeToNode = jest.fn(function (path, formState) {
        return formState;
      });

      _reactTestRenderer.default.create(React.createElement(_TestForm.default, {
        applyChangeToNode: applyChangeToNode
      }, React.createElement(_ObjectField.default, {
        link: link
      }, renderFn)));

      expect(applyChangeToNode).toHaveBeenCalledTimes(0);
      expect(link.onChange).toHaveBeenCalledTimes(0); // call the child onChange

      var objectLinks = renderFn.mock.calls[0][0];
      var newChildMeta = (0, _tools.mockFormState)("newString");
      objectLinks.string.onChange(newChildMeta);
      expect(applyChangeToNode).toHaveBeenCalledTimes(1);
      expect(applyChangeToNode).toHaveBeenCalledWith([], [{
        string: "newString",
        number: 42
      }, expect.anything()]);
      expect(link.onChange).toHaveBeenCalledTimes(1);
      expect(link.onChange).toHaveBeenCalledWith([{
        string: "newString",
        number: 42
      }, formState[1]]);
    });
    it("calls onBlur when a child is blurred", function () {
      var formStateInner = {
        string: "hello",
        number: 42
      };
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_ObjectField.default, {
        link: link
      }, renderFn));

      var objectLinks = renderFn.mock.calls[0][0]; // call the child onBlur

      var newChildTree = (0, _tools.mockFormState)("")[1];
      objectLinks.number.onBlur(newChildTree);
      expect(link.onBlur).toHaveBeenCalled();
      var newObjectTree = link.onBlur.mock.calls[0][0];
      expect(newObjectTree.children.number).toBe(newChildTree);
      expect(newObjectTree.data.meta).toMatchObject({
        touched: true,
        changed: false
      });
    });
    it("calls its own validation when a child changes", function () {
      var renderFn = jest.fn(function () {
        return null;
      });
      var validation = jest.fn(function () {
        return ["This is an error"];
      });

      _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          string: "hello",
          number: 42
        }
      }, function (link) {
        return React.createElement(_ObjectField.default, {
          link: link,
          validation: validation
        }, renderFn);
      }));

      expect(validation).toHaveBeenCalledTimes(1);
      var objectLinks = renderFn.mock.calls[0][0]; // call the child onChange

      var newChildMeta = (0, _tools.mockFormState)("newString");
      objectLinks.string.onChange(newChildMeta);
      expect(validation).toHaveBeenCalledTimes(2);
      expect(validation.mock.calls[1][0]).toEqual({
        string: "newString",
        number: 42
      });
    });
  });
  describe("customChange", function () {
    it("allows sibling fields to be overwritten", function () {
      var formStateInner = {
        string: "hello",
        number: 42
      };
      var formState = (0, _tools.mockFormState)(formStateInner);
      var link = (0, _tools.mockLink)(formState);
      var renderFn = jest.fn(function () {
        return null;
      });
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return {
          string: "A whole new value",
          number: 0
        };
      });

      _reactTestRenderer.default.create(React.createElement(_TestForm.default, null, React.createElement(_ObjectField.default, {
        link: link,
        validation: jest.fn(function () {
          return ["This is an error"];
        }),
        customChange: customChange
      }, renderFn)));

      var objectLinks = renderFn.mock.calls[0][0]; // call the child onChange

      var newChildMeta = (0, _tools.mockFormState)("newString");
      objectLinks.string.onChange(newChildMeta); // customChange should be called with the correct args

      expect(customChange).toHaveBeenCalledTimes(1);
      expect(customChange).toHaveBeenCalledWith({
        string: "hello",
        number: 42
      }, {
        string: "newString",
        number: 42
      }); // onChange should be called with the result of customChange

      expect(link.onChange).toHaveBeenCalledTimes(1);
      expect(link.onChange).toHaveBeenCalledWith([{
        string: "A whole new value",
        number: 0
      }, expect.anything()]);
    });
    it("can return null to signal there was no custom change", function () {
      var renderFn = jest.fn(function () {
        return null;
      });
      var validation = jest.fn(function () {
        return ["This is an error"];
      });
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return null;
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          string: "hello",
          number: 42
        }
      }, function (link) {
        return React.createElement(_ObjectField.default, {
          link: link,
          validation: validation,
          customChange: customChange
        }, renderFn);
      }));

      var objectLinks = renderFn.mock.calls[0][0]; // call the child onChange

      var newChildMeta = (0, _tools.mockFormState)("newString");
      objectLinks.string.onChange(newChildMeta);
      expect(customChange).toHaveBeenCalledTimes(1);
      var link = renderer.root.findByType(_ObjectField.default).instance.props.link; // the value we get out is as if customChange didn't exist

      expect(link.formState).toEqual([{
        string: "newString",
        number: 42
      }, expect.anything()]); // Validated the result of customChange

      expect(validation).toHaveBeenCalledTimes(2);
      expect(validation.mock.calls[1][0]).toEqual({
        string: "newString",
        number: 42
      });
    });
    it("doesn't break validations for child fields", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return {
          string: "a whole new value",
          string2: "modified sibling value"
        };
      });
      var childValidation = jest.fn(function () {
        return ["This is an error"];
      });
      var parentValidation = jest.fn(function () {
        return ["This is an error from the parent"];
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          string: "hello",
          string2: "goodbye"
        }
      }, function (link) {
        return React.createElement(_ObjectField.default, {
          link: link,
          customChange: customChange,
          validation: parentValidation
        }, function (links) {
          return React.createElement(React.Fragment, null, React.createElement(_TestField.default, {
            link: links.string,
            validation: childValidation
          }), React.createElement(_TestField.default, {
            link: links.string2,
            validation: childValidation
          }));
        });
      })); // after mount, validate everything


      expect(parentValidation).toHaveBeenCalledTimes(1);
      expect(childValidation).toHaveBeenCalledTimes(2); // Now change one of the values

      parentValidation.mockClear();
      childValidation.mockClear();
      var inner = renderer.root.findAllByType(_TestField.TestInput)[0];
      inner.instance.change("zach"); // Note that child validation will be called 5 times:
      // * Once in response to the change
      // * Twice more from the arrayEquals test in replaceValidation
      // * Twice more from recomputeErrorsAtPathAndRender

      expect(parentValidation).toHaveBeenCalledTimes(1);
      expect(childValidation).toHaveBeenCalledTimes(1 + 2 + 2);
      var link = renderer.root.findByType(_ObjectField.default).instance.props.link;
      expect(link.formState).toEqual([{
        string: "a whole new value",
        string2: "modified sibling value"
      }, expect.anything()]);
    });
    it("doesn't create a new instance (i.e. remount)", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return {
          string: "A whole new value",
          number: 0
        };
      });

      var renderer = _reactTestRenderer.default.create(React.createElement(_ObjectField.default, {
        link: (0, _tools.mockLink)((0, _tools.mockFormState)({
          string: "hello",
          number: 42
        })),
        customChange: customChange
      }, function (links) {
        return React.createElement(_TestField.default, {
          link: links.string
        });
      }));

      var testInstance = renderer.root.findAllByType(_TestField.TestInput)[0].instance; // now trigger a customChange, which used to cause a remount

      testInstance.change("hi");
      expect(customChange).toHaveBeenCalledTimes(1); // but we no longer cause a remount, so the instances should be the same

      var nextTestInstance = renderer.root.findAllByType(_TestField.TestInput)[0].instance; // Using Object.is here because toBe hangs as the objects are
      // self-referential and thus not printable

      expect(Object.is(testInstance, nextTestInstance)).toBe(true);
    });
    it("works fine even if not at the root of the form", function () {
      var customChange = jest.fn(function (_oldValue, _newValue) {
        return {
          string: "A whole new value",
          number: 0
        };
      });
      var objectRenderFn = jest.fn(function () {
        return null;
      });
      var linkTapFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          uncle: "Bob",
          nested: {
            string: "hello",
            number: 42
          }
        }
      }, function (link) {
        return React.createElement(_ObjectField.default, {
          link: link
        }, function (link) {
          return React.createElement(React.Fragment, null, React.createElement(_TestField.default, {
            link: link.uncle
          }), React.createElement(_LinkTap.default, {
            link: link.nested
          }, linkTapFn), React.createElement(_ObjectField.default, {
            link: link.nested,
            validation: jest.fn(function () {
              return ["This is an error"];
            }),
            customChange: customChange
          }, objectRenderFn));
        });
      }));

      linkTapFn.mockClear(); // call the child onChange to trigger a customChange

      var objectLinks = objectRenderFn.mock.calls[0][0];
      objectLinks.string.onChange((0, _tools.mockFormState)("newString")); // onChange should be called with the result of customChange

      expect(linkTapFn).toHaveBeenCalledTimes(2);
      expect(linkTapFn.mock.calls[0][0].formState).toEqual([{
        string: "A whole new value",
        number: 0
      }, expect.anything()]);
    });
    it("works fine with nested custom changes", function () {
      var outerCustomChange = jest.fn(function () {
        return {
          uncle: "Sam",
          nested: {
            string: "An even newer value",
            number: 1337
          }
        };
      });
      var innerCustomChange = jest.fn(function () {
        return {
          string: "A whole new value",
          number: 0
        };
      });
      var objectRenderFn = jest.fn(function () {
        return null;
      });
      var outerLinkTapFn = jest.fn(function () {
        return null;
      });
      var innerLinkTapFn = jest.fn(function () {
        return null;
      });

      _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          uncle: "Bob",
          nested: {
            string: "hello",
            number: 42
          }
        }
      }, function (link) {
        return React.createElement(React.Fragment, null, React.createElement(_LinkTap.default, {
          link: link
        }, outerLinkTapFn), React.createElement(_ObjectField.default, {
          link: link,
          customChange: outerCustomChange
        }, function (link) {
          return React.createElement(React.Fragment, null, React.createElement(_TestField.default, {
            link: link.uncle
          }), React.createElement(_LinkTap.default, {
            link: link.nested
          }, innerLinkTapFn), React.createElement(_ObjectField.default, {
            link: link.nested,
            validation: jest.fn(function () {
              return ["This is an error"];
            }),
            customChange: innerCustomChange
          }, objectRenderFn));
        }));
      }));

      innerLinkTapFn.mockClear();
      outerLinkTapFn.mockClear(); // call the child onChange to trigger a customChange

      var objectLinks = objectRenderFn.mock.calls[0][0];
      objectLinks.string.onChange((0, _tools.mockFormState)("newString"));
      expect(innerLinkTapFn).toHaveBeenCalledTimes(2);
      expect(innerLinkTapFn.mock.calls[0][0].formState).toEqual([{
        string: "An even newer value",
        number: 1337
      }, expect.anything()]);
      expect(outerLinkTapFn).toHaveBeenCalledTimes(2);
      expect(outerLinkTapFn.mock.calls[0][0].formState).toEqual([{
        uncle: "Sam",
        nested: {
          string: "An even newer value",
          number: 1337
        }
      }, expect.anything()]);
    });
    it("can change shape", function () {
      var renderer = _reactTestRenderer.default.create(React.createElement(_Form.default, {
        initialValue: {
          a: "a"
        }
      }, function (link) {
        return React.createElement(_ObjectField.default, {
          link: link,
          customChange: function customChange() {
            return {
              b: "b"
            };
          }
        }, function (link) {
          return React.createElement(React.Fragment, null, link.a ? React.createElement(_TestField.default, {
            link: link.a
          }) : React.createElement(_TestField.default, {
            link: link.b
          }));
        });
      }));

      var inner = renderer.root.findAllByType(_TestField.TestInput)[0];
      expect(function () {
        inner.instance.change("z");
      }).not.toThrow();
    });
  });
});