"use strict";

var _feedbackStrategies = _interopRequireWildcard(require("../feedbackStrategies"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

describe("feedbackStrategies", function () {
  describe("provided strategies", function () {
    describe("Always", function () {
      it("always returns true", function () {
        expect(_feedbackStrategies.default.Always()).toBe(true);
      });
    });
    describe("Touched", function () {
      it("returns true when the field is touched", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Touched(null, {
          touched: true
        })).toBe(true);
      });
      it("returns false when the field is not touched", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Touched(null, {
          touched: false
        })).toBe(false);
      });
    });
    describe("Changed", function () {
      it("returns true when the field is changed", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Changed(null, {
          changed: true
        })).toBe(true);
      });
      it("returns false when the field is not changed", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Changed(null, {
          changed: false
        })).toBe(false);
      });
    });
    describe("ClientValidationSucceeded", function () {
      it("returns true when the field has passed client validations in the past", function () {
        expect( // $FlowFixMe
        _feedbackStrategies.default.ClientValidationSucceeded(null, {
          succeeded: true
        })).toBe(true);
      });
      it("returns false when the field has not passed client validations in the past", function () {
        expect( // $FlowFixMe
        _feedbackStrategies.default.ClientValidationSucceeded(null, {
          succeeded: false
        })).toBe(false);
      });
    });
    describe("Pristine", function () {
      it("returns true when the form is pristine", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Pristine({
          pristine: true
        })).toBe(true);
      });
      it("returns false when the form is not pristine", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Pristine({
          pristine: false
        })).toBe(false);
      });
    });
    describe("Submitted", function () {
      it("returns true when the form is submitted", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Submitted({
          submitted: true
        })).toBe(true);
      });
      it("returns false when the form is not submitted", function () {
        // $FlowFixMe
        expect(_feedbackStrategies.default.Submitted({
          submitted: false
        })).toBe(false);
      });
    });
  });
  describe("combining strategies", function () {
    var t = function t() {
      return true;
    };

    var f = function f() {
      return false;
    };

    function callStrategy(strategy) {
      // $FlowFixMe
      return strategy();
    }

    describe("and()", function () {
      it("behaves like boolean conjunction", function () {
        expect(callStrategy((0, _feedbackStrategies.and)(t, t))).toBe(true);
        expect(callStrategy((0, _feedbackStrategies.and)(t, f))).toBe(false);
        expect(callStrategy((0, _feedbackStrategies.and)(f, t))).toBe(false);
        expect(callStrategy((0, _feedbackStrategies.and)(f, f))).toBe(false);
      });
    });
    describe("or()", function () {
      it("behaves like boolean disjunction", function () {
        expect(callStrategy((0, _feedbackStrategies.or)(t, t))).toBe(true);
        expect(callStrategy((0, _feedbackStrategies.or)(t, f))).toBe(true);
        expect(callStrategy((0, _feedbackStrategies.or)(f, t))).toBe(true);
        expect(callStrategy((0, _feedbackStrategies.or)(f, f))).toBe(false);
      });
    });
    describe("not()", function () {
      it("behaves like boolean negation", function () {
        expect(callStrategy((0, _feedbackStrategies.not)(t))).toBe(false);
        expect(callStrategy((0, _feedbackStrategies.not)(f))).toBe(true);
      });
    });
  });
});