"use strict";

var _tree = require("../tree");

var sampleTree = {
  type: "object",
  data: "root",
  children: {
    left: {
      type: "leaf",
      data: "left"
    },
    right: {
      type: "array",
      data: "right",
      children: [{
        type: "leaf",
        data: "zero"
      }, {
        type: "leaf",
        data: "one"
      }]
    }
  }
};
describe("tree", function () {
  it("can be mapped over", function () {
    var mappedTree = (0, _tree.mapTree)(function (s) {
      return s.length;
    }, sampleTree);
    var untypedRoot = mappedTree;
    expect(untypedRoot.type).toBe("object");
    expect(untypedRoot.data).toBe(4);
    var left = untypedRoot.children.left;
    expect(left.type).toBe("leaf");
    expect(left.data).toBe(4);
    var right = untypedRoot.children.right;
    expect(right.type).toBe("array");
    expect(right.data).toBe(5);
    var zero = right.children[0];
    expect(zero.type).toBe("leaf");
    expect(zero.data).toBe(4);
    var one = right.children[1];
    expect(one.type).toBe("leaf");
    expect(one.data).toBe(3);
  });
  it("can be folded into a monoid preorder", function () {
    var id = function id(x) {
      return x;
    };

    var folded = (0, _tree.foldMapTree)(id, "", function (x, y) {
      return x.concat(y);
    }, sampleTree);
    expect(folded).toBe("rootleftrightzeroone");
  });
});