"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.leaf = leaf;
exports.pathFromPathString = pathFromPathString;
exports.strictZipWith = strictZipWith;
exports.mapTree = mapTree;
exports.foldMapTree = foldMapTree;

var _set = require("./utils/set");

var _invariant = _interopRequireDefault(require("./utils/invariant"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function leaf(data) {
  return {
    type: "leaf",
    data: data
  };
}

function pathFromPathString(pathString) {
  if (pathString[0] !== "/") {
    throw new Error("Error paths must start with forward-slash");
  }

  if (pathString === "/") {
    return [];
  }

  return pathString.slice(1).split("/").map(function (keyPart) {
    // This might be dangerous, since it means you can't use numbers as object
    // keys. This is acceptable for now.
    if (!isNaN(keyPart)) {
      return {
        type: "array",
        index: Number.parseInt(keyPart, 10)
      };
    } else {
      return {
        type: "object",
        key: keyPart
      };
    }
  });
}

function strictZipWith(f, left, right) {
  if (left.type === "object" && right.type === "object") {
    if (!(0, _set.setEq)(new Set(Object.keys(left)), new Set(Object.keys(right)))) {
      throw new Error("Tried to zip two object nodes with different keys");
    }

    return {
      type: "object",
      data: f(left.data, right.data),
      children: Object.keys(left.children).reduce(function (memo, k) {
        (0, _invariant.default)(left.type === "object" && right.type === "object", "This should never happen.");
        return _objectSpread({}, memo, _defineProperty({}, k, strictZipWith(f, left.children[k], right.children[k])));
      }, {})
    };
  }

  if (left.type === "array" && right.type === "array") {
    if (left.children.length !== right.children.length) {
      throw new Error("Tried to zip two array nodes with different lengths");
    }

    return {
      type: "array",
      data: f(left.data, right.data),
      children: left.children.map(function (_, i) {
        (0, _invariant.default)(left.type === "array" && right.type === "array");
        return strictZipWith(f, left.children[i], right.children[i]);
      })
    };
  }

  if (left.type === "leaf" && right.type === "leaf") {
    return {
      type: "leaf",
      data: f(left.data, right.data)
    };
  }

  throw new Error("Tried to zip two nodes of different type");
} // A tree is a functor


function mapTree(f, tree) {
  if (tree.type === "object") {
    return {
      type: "object",
      data: f(tree.data),
      children: Object.keys(tree.children).reduce(function (memo, key) {
        return _objectSpread({}, memo, _defineProperty({}, key, mapTree(f, tree.children[key])));
      }, {})
    };
  } else if (tree.type === "array") {
    return {
      type: "array",
      data: f(tree.data),
      children: tree.children.map(function (child) {
        return mapTree(f, child);
      })
    };
  } else {
    return {
      type: "leaf",
      data: f(tree.data)
    };
  }
} // Fold a tree preorder


function foldMapTree(mapper, mempty, mappend, tree) {
  if (tree.type === "leaf") {
    return mapper(tree.data);
  } else if (tree.type === "array") {
    var foldedChildren = tree.children.reduce(function (memo, childTree) {
      return mappend(memo, foldMapTree(mapper, mempty, mappend, childTree));
    }, mempty);
    return mappend(mapper(tree.data), foldedChildren);
  } else {
    var _foldedChildren = Object.keys(tree.children).reduce(function (memo, key) {
      return mappend(memo, foldMapTree(mapper, mempty, mappend, tree.children[key]));
    }, mempty);

    return mappend(mapper(tree.data), _foldedChildren);
  }
}